#!/usr/bin/env ruby

require "pry"
# require "sinatra"
require "sinatra/cookies"
require 'sinatra/base'
require 'sinatra/namespace'
require "slim"
require "bcrypt"
require "jwt"
require "sequel"
require "json"
require "doc_my_routes"

require_relative "./utils.rb"

module LifePex
  DB = Sequel.connect ARGV[0] || ENV["LIFEPEX_DB"] || "sqlite://sqlite.db"
  BASE_URL = ARGV[1] || ENV["LIFEPEX_BASE_URL"] || "http://localhost:4567"
  SECRET = ENV['LIFEPEX_SECRET'] || "LifePexSecret"
  CODE_VERSION = DB[:meta].first[:code_version]
  CODE_DATE = DB[:meta].first[:code_date]
  include LifePex::Utils
end

require_relative "./json_api.rb"

require_relative "./models/level.rb"
require_relative "./models/user.rb"
require_relative "./models/pex.rb"
require_relative "./models/user_pex.rb"

require_relative "./auth.rb"
require_relative "./users.rb"
require_relative "./pex.rb"
require_relative "./pex2.rb"

class PublicSystem < Sinatra::Base
  set :public_folder, 'public'
end

class BaseSystem < Sinatra::Base
  DocMyRoutes.configure do |config|
    config.title = "LifePex"
    config.description = "LifePex JSON REST API documentation"
  end

  enable :sessions
  use AuthSystem
  use PublicSystem
  use UserSystem
  use PexSystem
  use Pex2System

  include JSON::API
  not_found do
    if accept_json?
      { message: "No route or entity \"#{request.path}\" found" }.to_json
    else
      raise
    end
  end

  run! if app_file == $0
end
